'''
Modified OCt 1st, 2023

@designed by: Adolfo Poma, Panagiotis Theodorakis and Marek Cieplak
'''

# 1-4 we use EN
# Everything further away than 4 has EN with LJ potantial 3.333 if it is not a Go contact
# Go contacts for anything further away than 4 according to the contact map.
# contacts for the EN are checked with all atoms and NOT only with the P atoms

import sys
from itertools import groupby
from math import sqrt

filename1 = sys.argv[1]     # 1st argument: the PDB datafile
ENcutoff = sys.argv[2]     # 2nd argument: the cutoff R_c for the EN = vdW + vdW + R_c
DNAchoice = sys.argv[3]    # 3rd argument: the choice of DNA (1: A-DNA, 2:Z-DNA, 3: B-DNA) 

alpha=pow(26/7.0, 1.0/6.0)  # augmentation factor for the overlap criterion

aminoacids = {
         #RES       Number of heavy atoms in every aminoacid
         'DA':     10,
         'DC':     8,
         'DG':     11,
         'DT':     9
         }
# Values from Table 4, Nucleic Acids Research 29, 3368 (2001)
if DNAchoice == '1':   
    vdwRadius = {
                # RES    ATOM  VDW radius
                  ('DA', 'C2'):2.83,
                  ('DA', 'C4'):2.04,
                  ('DA', 'C5'):2.08,
                  ('DA', 'C6'):2.00,
                  ('DA', 'C8'):2.76,
                  ('DA', 'N1'):2.27,
                  ('DA', 'N3'):2.37,
                  ('DA', 'N6'):2.85,
                  ('DA', 'N7'):2.41,
                  ('DA', 'N9'):1.85,
                  #
                  ('DC', 'C2'):2.11,
                  ('DC', 'C4'):1.91,
                  ('DC', 'C5'):2.77,
                  ('DC', 'C6'):2.65,
                  ('DC', 'N1'):1.84,
                  ('DC', 'N3'):2.38,
                  ('DC', 'N4'):2.83,
                  ('DC', 'O2'):2.43,
                  #
                  ('DG', 'C2'):1.96,
                  ('DG', 'C4'):2.02,
                  ('DG', 'C5'):2.08,
                  ('DG', 'C6'):2.15,
                  ('DG', 'C8'):2.81,
                  ('DG', 'N1'):2.44,
                  ('DG', 'N2'):2.87,
                  ('DG', 'N3'):2.41,
                  ('DG', 'N7'):2.43,
                  ('DG', 'N9'):1.83,
                  ('DG', 'O6'):2.45,
                  #
                  ('DT', 'C2'):2.09,
                  ('DT', 'C4'):2.13,
                  ('DT', 'C5'):1.81,
                  ('DT', 'C5M'):3.35,
                  ('DT', 'C6'):2.68,
                  ('DT', 'N1'):1.83,
                  ('DT', 'N3'):2.44,
                  ('DT', 'O2'):2.53,
                  ('DT', 'O4'):2.38
                  }
elif DNAchoice == '2':
        vdwRadius = {
                # RES    ATOM  VDW radius
                  ('DA', 'C2'):2.85,
                  ('DA', 'C4'):1.95,
                  ('DA', 'C5'):2.05,
                  ('DA', 'C6'):2.00,
                  ('DA', 'N1'):2.30,
                  ('DA', 'N3'):2.33,
                  ('DA', 'N6'):2.42,
                  ('DA', 'N7'):1.80,
                  #
                  ('DC', 'C2'):2.13,
                  ('DC', 'C4'):1.89,
                  ('DC', 'C5'):2.73,
                  ('DC', 'C6'):2.69,
                  ('DC', 'N1'):1.84,
                  ('DC', 'N3'):2.41,
                  ('DC', 'N4'):2.86,
                  ('DC', 'O2'):2.46,
                  #
                  ('DG', 'C2'):1.94,
                  ('DG', 'C4'):1.99,
                  ('DG', 'C5'):2.08,
                  ('DG', 'C6'):2.17,
                  ('DG', 'C8'):2.88,
                  ('DG', 'N1'):2.45,
                  ('DG', 'N2'):2.87,
                  ('DG', 'N3'):2.41,
                  ('DG', 'N7'):2.51,
                  ('DG', 'N9'):1.83,
                  ('DG', 'O6'):2.53,
                  #
                  ('DT', 'C2'):2.10,
                  ('DT', 'C4'):2.12,
                  ('DT', 'C5'):1.76,
                  ('DT', 'C6'):2.73,
                  ('DT', 'N1'):1.85,
                  ('DT', 'N3'):2.46,
                  ('DT', 'O2'):2.54
                  }
elif DNAchoice == '3':
        vdwRadius = {
                # RES    ATOM  VDW radius
                  ('DA', 'C2'):2.79,
                  ('DA', 'C4'):2.02,
                  ('DA', 'C5'):2.05,
                  ('DA', 'C6'):2.00,
                  ('DA', 'C8'):2.73,
                  ('DA', 'N1'):2.27,
                  ('DA', 'N3'):2.35,
                  ('DA', 'N6'):2.87,
                  ('DA', 'N7'):2.41,
                  ('DA', 'N9'):1.82,
                  #
                  ('DC', 'C2'):2.14,
                  ('DC', 'C4'):1.91,
                  ('DC', 'C5'):2.75,
                  ('DC', 'C6'):2.64,
                  ('DC', 'N1'):1.84,
                  ('DC', 'N3'):2.39,
                  ('DC', 'N4'):2.84,
                  ('DC', 'O2'):2.43,
                  #
                  ('DG', 'C2'):1.97,
                  ('DG', 'C4'):2.04,
                  ('DG', 'C5'):2.08,
                  ('DG', 'C6'):2.14,
                  ('DG', 'C8'):2.75,
                  ('DG', 'N1'):2.44,
                  ('DG', 'N2'):2.83,
                  ('DG', 'N3'):2.44,
                  ('DG', 'N7'):2.46,
                  ('DG', 'N9'):1.83,
                  ('DG', 'O6'):2.43,
                  #
                  ('DT', 'C2'):2.10,
                  ('DT', 'C4'):2.13,
                  ('DT', 'C5'):1.80,
                  ('DT', 'C5M'):3.32,
                  ('DT', 'C6'):2.67,
                  ('DT', 'N1'):1.82,
                  ('DT', 'N3'):2.45,
                  ('DT', 'O2'):2.49,
                  ('DT', 'O4'):2.48
                  }
else:
    print ("Wrong choice of DNA!!!")
    exit()
    
    
# 
AtomRecord = []
with open(filename1) as f:
    for a in f:
        line = a.strip()
        if line.startswith("ATOM") or line.startswith("HETATM"):
            AtomRecord.append((line[12:16].strip(),line[17:20].strip(),int(line[22:26].strip()),line[21].strip(), float(line[30:38].strip()),float(line[38:46].strip()),float(line[46:54].strip())))
        ## ===> atom name,       res name,        res id,                        chain,       x             y                z
        #AtomRecord = (a[12:16].strip(),a[17:20].strip(),int(a[22:26])+(ord(a[26])<<20),a[21], float(a[30:38]),float(a[38:46]),float(a[46:54]))
        
#for i in AtomRecord:
#    print i
#print AtomRecord
ResList = []
uniquekeys = []
for k, g in groupby(AtomRecord, lambda x: x[2]):
    ResList.append(list(g))
    uniquekeys.append(k)
# for i in range(0,len(ResList)):
#     print ResList[i]

# the P atoms as it is based on the P model
calphaAtoms = []
for residue in ResList:
    for atom in residue:
        if atom[0] == 'P':
            calphaAtoms.append(atom)   
# for ii in calphaAtoms:
#     print ii
                     
     
goContacts = []
enContacts = []
for i in range(0,len(ResList)-1):
    for j in range(i+1,len(ResList)):
        foundgo = 0
        foundEN = 0
        foundENLJ = 0
        if j-i > 3:  # for Go contacts
            for k in range(0,len(ResList[i])):
                for l in range(0,len(ResList[j])):
                    distx = (ResList[i][k][4] - ResList[j][l][4]) * (ResList[i][k][4] - ResList[j][l][4]) 
                    disty = (ResList[i][k][5] - ResList[j][l][5]) * (ResList[i][k][5] - ResList[j][l][5])
                    distz = (ResList[i][k][6] - ResList[j][l][6]) * (ResList[i][k][6] - ResList[j][l][6])
                    dist = sqrt(distx + disty + distz)
                    if (ResList[i][k][1],ResList[i][k][0]) in vdwRadius and (ResList[j][l][1],ResList[j][l][0]) in vdwRadius: 
                        if dist <= alpha * (vdwRadius[ResList[i][k][1],ResList[i][k][0]] + vdwRadius[ResList[j][l][1],ResList[j][l][0]]):
                            foundgo = 1
                            #print i, k, j, l, ResList[i][k][1], ResList[i][k][0],ResList[j][l][1], ResList[j][l][0]  
                            distx = (calphaAtoms[i][4] - calphaAtoms[j][4]) * (calphaAtoms[i][4] - calphaAtoms[j][4])
                            disty = (calphaAtoms[i][5] - calphaAtoms[j][5]) * (calphaAtoms[i][5] - calphaAtoms[j][5])
                            distz = (calphaAtoms[i][6] - calphaAtoms[j][6]) * (calphaAtoms[i][6] - calphaAtoms[j][6])
                            dist = sqrt(distx + disty + distz)
                            distgo = dist/1.12246
        if foundgo == 1:
            mycontacts = [i,j,distgo]   # i,j residues form a contacts
            goContacts.append(mycontacts)
        elif j-i < 4 and foundgo == 0:
            for k in range(0,len(ResList[i])):
                for l in range(0,len(ResList[j])):
                    distx = (ResList[i][k][4] - ResList[j][l][4]) * (ResList[i][k][4] - ResList[j][l][4]) 
                    disty = (ResList[i][k][5] - ResList[j][l][5]) * (ResList[i][k][5] - ResList[j][l][5])
                    distz = (ResList[i][k][6] - ResList[j][l][6]) * (ResList[i][k][6] - ResList[j][l][6])
                    dist = sqrt(distx + disty + distz)
#                    print item11, item12, item21, item22
#                    print vdwRadius[item11,item12], vdwRadius[item21,item22]
                    if (ResList[i][k][1],ResList[i][k][0]) in vdwRadius and (ResList[j][l][1],ResList[j][l][0]) in vdwRadius:
                        if dist <= (vdwRadius[ResList[i][k][1],ResList[i][k][0]] + vdwRadius[ResList[j][l][1],ResList[j][l][0]] + float(ENcutoff)):
                            foundEN = 1
                            distx = (calphaAtoms[i][4] - calphaAtoms[j][4]) * (calphaAtoms[i][4] - calphaAtoms[j][4])
                            disty = (calphaAtoms[i][5] - calphaAtoms[j][5]) * (calphaAtoms[i][5] - calphaAtoms[j][5])
                            distz = (calphaAtoms[i][6] - calphaAtoms[j][6]) * (calphaAtoms[i][6] - calphaAtoms[j][6])
                            dist = sqrt(distx + disty + distz)
                            distEN = dist
        elif j-i > 3 and foundgo == 0:
            for k in range(0,len(ResList[i])):
                for l in range(0,len(ResList[j])):
                    distx = (ResList[i][k][4] - ResList[j][l][4]) * (ResList[i][k][4] - ResList[j][l][4]) 
                    disty = (ResList[i][k][5] - ResList[j][l][5]) * (ResList[i][k][5] - ResList[j][l][5])
                    distz = (ResList[i][k][6] - ResList[j][l][6]) * (ResList[i][k][6] - ResList[j][l][6])
                    dist = sqrt(distx + disty + distz)
#                    print item11, item12, item21, item22
#                    print vdwRadius[item11,item12], vdwRadius[item21,item22]
                    if (ResList[i][k][1],ResList[i][k][0]) in vdwRadius and  (ResList[j][l][1],ResList[j][l][0]) in vdwRadius:
                        if dist <= (vdwRadius[ResList[i][k][1],ResList[i][k][0]] + vdwRadius[ResList[j][l][1],ResList[j][l][0]] + float(ENcutoff)):
                            foundENLJ = 1
                            distx = (calphaAtoms[i][4] - calphaAtoms[j][4]) * (calphaAtoms[i][4] - calphaAtoms[j][4])
                            disty = (calphaAtoms[i][5] - calphaAtoms[j][5]) * (calphaAtoms[i][5] - calphaAtoms[j][5])
                            distz = (calphaAtoms[i][6] - calphaAtoms[j][6]) * (calphaAtoms[i][6] - calphaAtoms[j][6])
                            dist = sqrt(distx + disty + distz)
                            distENLJ = dist/1.12246
            
        if foundEN == 1:
            mycontacts=[i,j,distEN]
            enContacts.append(mycontacts)
        if foundENLJ == 1:
            mycontacts = [i,j,distENLJ]
            goContacts.append(mycontacts)
            
# 
# print "gocontacts"
# print goContacts
# print "EN contacts"
# print enContacts


# write topology file
filename2 = open('topol.top','w')
filename2.write('[ defaults ]' + "\n")
filename2.write('1 2' + "\n\n")
filename2.write('[ atomtypes ]' + "\n")
filename2.write('P 1.0 0.00 A 0.00 0.00' + "\n\n")

filename2.write('[ moleculetype ]' + "\n")
filename2.write('NucleicAcid      0' + "\n\n")



filename2.write('[ atoms ]' + "\n")
for i in range(0,len(calphaAtoms)):
    filename2.write(str(i+1) + " " +  calphaAtoms[i][0] + " " + str(calphaAtoms[i][2]) + " " + calphaAtoms[i][1] + " P " + str(i+1) + " 0.000 1.000" + "\n")
filename2.write("\n")
# print calphaAtoms

filename2.write('[ bonds ]' + "\n")
for thisENContact in enContacts:
    filename2.write(str(thisENContact[0]+1) + " " + str(thisENContact[1]+1) + " 1 " + str(thisENContact[2]/10.0)[:6] + " 100.0" + "\n")

# print calphaAtoms
filename2.write("\n" + '[ pairs ]' + "\n")
for thisGoContact in goContacts:
#    print thisGoContact
    filename2.write(str(thisGoContact[0]+1) + " " + str(thisGoContact[1]+1) + " 1 " + str(thisGoContact[2]/10.0)[:6] + " 3.333" + "\n")

filename2.write("\n" + '[ system ]' + "\n")
filename2.write('goDNA model for ' + filename1[:-4] + "\n\n")

filename2.write('[ molecules ]' + "\n")
filename2.write('NucleicAcid        1' + "\n")

filename2.close()

# write gro file
groFile = filename1[:-4]+".gro"
filenamegro = open(groFile,'w')
filenamegro.write('goDNA model for ' + filename1[:-4] + "\n")
filenamegro.write(str(len(calphaAtoms)) + "\n")
for i in range(0,len(calphaAtoms)):
    mystring = '%5d%5s%4s%5d%8.3f%8.3f%8.3f\n' % (i+1, calphaAtoms[i][1], calphaAtoms[i][0], i+1, calphaAtoms[i][4]/10.0, calphaAtoms[i][5]/10.0, calphaAtoms[i][6]/10.0  )
    filenamegro.write(mystring)
mybox = " 100.000 100.0 100.000" + "\n"
filenamegro.write(mybox)


if __name__ == '__main__':
    pass
